using System;
using System.Reflection;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;
using System.Threading;
using Microsoft.FlightSimulator.SimConnect;
using System.IO;
using System.Drawing;
using System.Drawing.Imaging;
using System.Drawing.Drawing2D;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Resources;

namespace Autothumbnail
{
    /**
     * TODO: travail sur simstart
     */
    class Autothumbnail
    {
        enum ScreenType
        {
            SINGLE,
            ALL,
            ALL_NO_OVERWRITE
        }

        private SimConnect simconnect;
        private EventWaitHandle waiter;

        // base config
        private string screensDirectory;

        // screen/ac data
        private AircraftTitle acTitle;
        private string acAirFile;

        private DateTime lastScreenShot = DateTime.MinValue;
        private ScreenType screenType;

        private bool menuCreated = false;

        // **************************************************************************
        //
        // SimConnect IDs
        //
        // **************************************************************************


        enum EVENTS : long
        {
            SCREEN_CAPTURE,
            SECOND,
            SIM_START,
            AC_CHANGED,
            MENU_ITEM,
            MENU_ITEM_CREATE_SINGLE,
            MENU_ITEM_CREATE_ALL,
            MENU_ITEM_CREATE_ALL_NO_OVERWRITE,
            UNUSED = 0xffffffff,
        }

        enum REQUESTS
        {
            REQUEST_AC_TITLE, 
            REQUEST_AIR_FILE,
            REQUEST_SIM_STATE,
        }

        enum DEFINITIONS
        {
            DEFINITION_AC_TITLE,
        }


        [StructLayout(LayoutKind.Sequential, CharSet = CharSet.Ansi, Pack = 1)]
        struct AircraftTitle
        {
            [MarshalAs(UnmanagedType.ByValTStr, SizeConst = 256)]
            public String title;
        }


        // **************************************************************************
        //
        // Initialisation / configuration
        //
        // **************************************************************************


        public Autothumbnail(uint simCoConfIndex)
        {
            //
            // set up simconnect
            //
            waiter = new EventWaitHandle(true, EventResetMode.AutoReset);
            try
            {
                simconnect = new SimConnect("Autothumbnail", IntPtr.Zero, 0x0402, waiter, simCoConfIndex);
            }
            catch (COMException cex)
            {
                displayError(Strings.String1, cex.Message);
            }


            // add simconnect events
            simconnect.OnRecvOpen += new SimConnect.RecvOpenEventHandler(OnSimConnectOpen);
            simconnect.OnRecvException += new SimConnect.RecvExceptionEventHandler(OnSimConnectException);
            simconnect.OnRecvQuit += new SimConnect.RecvQuitEventHandler(OnSimConnectQuit);
            simconnect.OnRecvEventFilename += new SimConnect.RecvEventFilenameEventHandler(OnSimConnectEventFileName);
            simconnect.OnRecvEvent += new SimConnect.RecvEventEventHandler(OnSimConnectEvent);
            simconnect.OnRecvSimobjectData += new SimConnect.RecvSimobjectDataEventHandler(OnSimConnectData);
            simconnect.OnRecvSystemState += new SimConnect.RecvSystemStateEventHandler(OnSimConnectSystemState);

        }

        private void InitRequests()
        {
            try
            {
                simconnect.AddToDataDefinition(DEFINITIONS.DEFINITION_AC_TITLE, "title", null,
                    SIMCONNECT_DATATYPE.STRING256, 0, SimConnect.SIMCONNECT_UNUSED);
                simconnect.RegisterDataDefineStruct<AircraftTitle>(DEFINITIONS.DEFINITION_AC_TITLE);

                // with this event we will know when the user change aircraft
                simconnect.SubscribeToSystemEvent(EVENTS.AC_CHANGED, "AircraftLoaded");
                simconnect.SetSystemEventState(EVENTS.AC_CHANGED, SIMCONNECT_STATE.ON);

                // subscribe to some basic system events.
                // 1 sec will be used as a timer to delay the reading of the thumbnail
                // simstart will be used to create menu (creating them just upon open
                // can cause ctd)
                simconnect.SubscribeToSystemEvent(EVENTS.SECOND, "1sec");
                simconnect.SubscribeToSystemEvent(EVENTS.SIM_START, "SimStart");
                simconnect.SetSystemEventState(EVENTS.SECOND, SIMCONNECT_STATE.ON);
                simconnect.SetSystemEventState(EVENTS.SIM_START, SIMCONNECT_STATE.ON);

                simconnect.MapClientEventToSimEvent(EVENTS.SCREEN_CAPTURE, "CAPTURE_SCREENSHOT");


                // request the name of the .air file. Normally we get notified of loading,
                // but the aircraft selected in the menu is loaded BEFORE we get up and
                // so we don't receive the event.
                // Thanks to ziporama who reminded me of this function
                simconnect.RequestSystemState(REQUESTS.REQUEST_AIR_FILE, 
                    "AircraftLoaded");
                simconnect.RequestSystemState(REQUESTS.REQUEST_SIM_STATE,
                    "Sim");

            }
            catch (COMException cex)
            {
                displayError(cex.Message);
            }
        }


        private void InitMenu(SimConnect simconnect)
        {
            if (!menuCreated)
            {
                menuCreated = true;
                displayText("Creating menu");
                try
                {
                    simconnect.MenuAddItem(Strings.Menu, EVENTS.MENU_ITEM, 1);
                    simconnect.MenuAddSubItem(EVENTS.MENU_ITEM, Strings.CurrentTexture, EVENTS.MENU_ITEM_CREATE_SINGLE, 0);
                    simconnect.MenuAddSubItem(EVENTS.MENU_ITEM, Strings.AllTextures, EVENTS.MENU_ITEM_CREATE_ALL, 0);
                    simconnect.MenuAddSubItem(EVENTS.MENU_ITEM, Strings.AllTexturesNowrite, EVENTS.MENU_ITEM_CREATE_ALL_NO_OVERWRITE, 0);
                }
                catch (COMException)
                {
                }
            }

        }

        /// <summary>
        /// Finds the directory where FSX stores screenshots (My Documents/My Pictures/...)
        /// </summary>
        /// <returns>dir, null if not found</returns>
        private string FindScreenShotsDirectory()
        {
            string myPicturesDir = System.Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);
            if (myPicturesDir == null) return null;

            // list contents
            DirectoryInfo di = new DirectoryInfo(myPicturesDir);
            DirectoryInfo[] subdirs = di.GetDirectories();
            foreach (DirectoryInfo si in subdirs)
            {
                if (si.Name.ToLower().Contains("flight simulator x"))
                {
                    // found FSX my pictures dir
                    return si.FullName;
                }
            }
            return null;    // fallback, not found
        }

        /// <summary>
        /// 
        /// Find the texture path corresponding to the currently used aircraft
        /// </summary>
        /// <returns></returns>
        private string FindTexturePath()
        {
            if (acAirFile == null ||
                acTitle.title == null ||
                acTitle.title.Equals("")) return null;


            // find aircraft cfg
            DirectoryInfo acDir = new FileInfo(acAirFile).Directory;
            FileInfo[] acFiles = acDir.GetFiles("aircraft.cfg");
            if (acFiles == null || acFiles.Length == 0)
                return null;    // doh! no aircraft.cfg?

            StreamReader sr = new StreamReader(acFiles[0].FullName);
            bool currentTitle = false;
            string currentTextureDir = null;
            string line;
            while ((line = sr.ReadLine()) != null)
            {
                line = line.Trim().ToLower();
                if (line.StartsWith("[fltsim"))
                {
                    // reset find flags
                    currentTitle = false;
                    currentTextureDir = null;
                    continue;
                }

                if (line.StartsWith("title"))
                {
                    int ip = line.IndexOf('=');
                    if (ip != -1)
                    {
                        line = line.Substring(ip + 1).Trim();
                        if (line.Equals(acTitle.title, StringComparison.CurrentCultureIgnoreCase))
                            currentTitle = true;
                    }
                }

                if (line.StartsWith("texture"))
                {
                    int ip = line.IndexOf('=');
                    if (ip != -1) currentTextureDir = line.Substring(ip + 1).Trim();
                }

                if (currentTitle &&
                    currentTextureDir != null)
                {

                    // found texture
                    sr.Close();

                    // strip "" and ''
                    // Thanks to many people who pointed out a stupid bug here
                    if (currentTextureDir.Length > 1 &&
			            currentTextureDir[0] == '\"' &&
                        currentTextureDir[currentTextureDir.Length - 1] == '\"')
                    {
                        currentTextureDir = currentTextureDir.Substring(1, currentTextureDir.Length - 2);
                    }
                    if (currentTextureDir.Length > 1 &&
		            	currentTextureDir[0] == '\'' &&
                        currentTextureDir[currentTextureDir.Length - 1] == '\'')
                    {
                        currentTextureDir = currentTextureDir.Substring(1, currentTextureDir.Length - 2);
                    }

                    return acDir.FullName + "\\texture" +
                        ((currentTextureDir.Length > 0) ? ("." + currentTextureDir) : "");
                }
            }
            sr.Close();
            return null;    // fallback
        }


        // **************************************************************************
        //
        // Simconnect events
        //
        // **************************************************************************


        /// <summary>
        /// 
        /// 
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void OnSimConnectOpen(SimConnect sender, SIMCONNECT_RECV_OPEN e)
        {
            displayText("Connected to " + e.szApplicationName);

            // initialize base simconnect requests
            InitRequests();
        }

        private void OnSimConnectException(SimConnect sender, SIMCONNECT_RECV_EXCEPTION e)
        {
            displayText("SimConnect exception (" + e.dwException + ") packet " + e.dwSendID);
        }

        private void OnSimConnectQuit(SimConnect sender, SIMCONNECT_RECV e)
        {
            if (simconnect != null)
            {
                simconnect.Dispose();
            }
            // exit gracefully
            System.Environment.Exit(0);
        }

        private void OnSimConnectEventFileName(SimConnect sender, SIMCONNECT_RECV_EVENT_FILENAME ev)
        {
            if (ev.uEventID == (uint)EVENTS.AC_CHANGED)
            {
                displayText("Aircraft changed to : " + ev.szFileName);
                acAirFile = ev.szFileName;
                acTitle.title = null;

                // request aircraft title to complete our informations
                try
                {
                    simconnect.RequestDataOnSimObject(REQUESTS.REQUEST_AC_TITLE,
                        DEFINITIONS.DEFINITION_AC_TITLE,
                        0, SIMCONNECT_PERIOD.ONCE,
                        SIMCONNECT_DATA_REQUEST_FLAG.DEFAULT, 0, 0, 0);
                }
                catch (COMException) { }

            }
        }

        private void OnSimConnectEvent(SimConnect sender, SIMCONNECT_RECV_EVENT e)
        {
            if (e.uEventID == (uint)EVENTS.MENU_ITEM_CREATE_SINGLE ||
                e.uEventID == (uint)EVENTS.MENU_ITEM_CREATE_ALL ||
                e.uEventID == (uint)EVENTS.MENU_ITEM_CREATE_ALL_NO_OVERWRITE)
            {
                //
                // set appropriate shoot mode
                if (e.uEventID == (uint)EVENTS.MENU_ITEM_CREATE_SINGLE)
                    screenType = ScreenType.SINGLE;
                else if (e.uEventID == (uint)EVENTS.MENU_ITEM_CREATE_ALL)
                    screenType = ScreenType.ALL;
                else
                    screenType = ScreenType.ALL_NO_OVERWRITE;

                //
                // send screenshot capture command
                //
                try
                {
                    simconnect.TransmitClientEvent(0, EVENTS.SCREEN_CAPTURE, 1,
                        (Enum)Enum.ToObject(typeof(EVENTS), SimConnect.SIMCONNECT_GROUP_PRIORITY_HIGHEST),
                        SIMCONNECT_EVENT_FLAG.GROUPID_IS_PRIORITY);
                    // mark screenshot time
                    lastScreenShot = DateTime.Now;
                }
                catch (COMException cex)
                {
                    displayError(Strings.String2, cex.Message);
                }

            }
            else if (e.uEventID == (uint)EVENTS.SECOND)
            {
                // every second event.
                // this enable us to wait a little before opening the screenshot image file
                // since it can be delayed by FSX I/O thread

                // no screenshot take
                if (lastScreenShot.Equals(DateTime.MinValue)) return;

                TimeSpan ts = DateTime.Now - lastScreenShot;
                if (ts.TotalMilliseconds > 1000)
                {
                    // ok we're ready
                    ProcessScreenShot();
                    lastScreenShot = DateTime.MinValue; // reset value


                }
            }
            else if (e.uEventID == (uint)EVENTS.SIM_START)
            {
                // create menu if not yet done
                InitMenu(sender);
            }
        }

        private void OnSimConnectData(SimConnect sender, SIMCONNECT_RECV_SIMOBJECT_DATA e)
        {
            if (e.dwRequestID == (uint)REQUESTS.REQUEST_AC_TITLE)
            {
                // received aircraft container title
                acTitle = (AircraftTitle)e.dwData[0];
            }
        }

        private void OnSimConnectSystemState(SimConnect sender, SIMCONNECT_RECV_SYSTEM_STATE e)
        {
            if (e.dwRequestID == (uint)REQUESTS.REQUEST_AIR_FILE)
            {
                displayText("Aircraft changed to : " + e.szString);
                acAirFile = e.szString;
                acTitle.title = null;

                // request aircraft title to complete our informations
                try
                {
                    simconnect.RequestDataOnSimObject(REQUESTS.REQUEST_AC_TITLE,
                        DEFINITIONS.DEFINITION_AC_TITLE,
                        0, SIMCONNECT_PERIOD.ONCE,
                        SIMCONNECT_DATA_REQUEST_FLAG.DEFAULT, 0, 0, 0);
                }
                catch (COMException) { }
            }
            else if (e.dwRequestID == (uint)REQUESTS.REQUEST_SIM_STATE)
            {
                if (e.dwInteger == 1)
                {
                    // create menu if not yet done
                    InitMenu(sender);
                }
            }
            
        }

        // **************************************************************************
        //
        // Find screenshot & copy
        //
        // **************************************************************************


        private bool ProcessScreenShot()
        {
            if (screensDirectory == null)
                screensDirectory = FindScreenShotsDirectory();

            if (screensDirectory == null)
            {
                // bad we didn't found it
                displayError(Strings.String3);
                return false;
            }

            // find the last screenshot
            string screenFile = FindScreenShot();
            if (screenFile == null) return false;

            // ok, now process bitmap
            Image thumb = MakeThumbnail(screenFile);
            if (thumb == null)
            {
                displayError(Strings.String4);
                return false;
            }

            // delete file since we made a thumbnail
            try
            {
                new FileInfo(screenFile).Delete();
            }
            catch (Exception) { /* don't really care */ }

            // create JPG encoder and parameters
            ImageCodecInfo[] icit = ImageCodecInfo.GetImageEncoders();
            ImageCodecInfo jpgCi = null;
            foreach (ImageCodecInfo ici in icit)
            {
                if (ici.FilenameExtension.ToLower().Contains("jpg"))
                    jpgCi = ici;
            }
            if (jpgCi == null)
            {
                displayError(Strings.String5);
                return false;
            }

            EncoderParameters par = new EncoderParameters(1);
            par.Param[0] = new EncoderParameter(System.Drawing.Imaging.Encoder.Quality, 50L);

            //
            // save thumbnail to one path of multiple if selected
            if (screenType == ScreenType.SINGLE)
            {
                //  find texture dir
                string txtDir = FindTexturePath();
                if (txtDir == null)
                {
                    displayError(Strings.String6, acAirFile, acTitle.title);
                    return false;
                }

                if (!SaveThumbnail(thumb, txtDir, jpgCi, par))
                {
                    displayError(Strings.String7, txtDir);
                }
            }
            else
            {
                DirectoryInfo acDir = new FileInfo(acAirFile).Directory;
                DirectoryInfo[] txtDirs = acDir.GetDirectories("texture*");
                if (txtDirs == null || txtDirs.Length == 0)
                {
                    // no texture to write
                    return false;
                }
                foreach (DirectoryInfo txtDir in txtDirs)
                {
                    if (screenType == ScreenType.ALL_NO_OVERWRITE)
                    {
                        // check if there is a thumbnail.jpg
                        FileInfo[] existingThumbs = txtDir.GetFiles("thumbnail.jpg");
                        if (existingThumbs != null &&
                            existingThumbs.Length > 0)
                            continue;       // proceed to next dir
                    }

                    if (!SaveThumbnail(thumb, txtDir.FullName, jpgCi, par))
                    {
                        displayError(Strings.String7, txtDir);
                        return false;   // break at first failure
                    }
                }
            }

            thumb.Dispose();
            return true;    // ok, all went fine

        }

        /// <summary>
        /// Find the screen shot image file whose time stamp is the nearest
        /// to the date/time we triggered the screen event
        /// </summary>
        /// <returns></returns>
        private string FindScreenShot()
        {
            DirectoryInfo screensDir = new DirectoryInfo(screensDirectory);
            if (screensDir == null || !screensDir.Exists)
                return null;

            FileInfo foundFile = null;
            double foundFileTimeDiff = Double.MaxValue;

            foreach (FileInfo screenFile in screensDir.GetFiles())
            {
                // split the file into pieces
                string[] parts = screenFile.Name.Split('-', '_', '.');
                if (parts.Length < 6) continue;

                DateTime fileTime = DateTime.MinValue;
                try {
                    fileTime = new DateTime(Int32.Parse(parts[0]),
                                        Int32.Parse(parts[1]),
                                        Int32.Parse(parts[2]),
                                        Int32.Parse(parts[3]),
                                        Int32.Parse(parts[4]),
                                        Int32.Parse(parts[5]));
                } catch (FormatException) {
                    // don't care 
                    continue;
                }

                if (fileTime == DateTime.MinValue) continue;

                // calculate time span between this file and the
                // reference screen shot file time
                //
                // we'll keep only the nearest file, but no more than 30
                // seconds away
                double fileTimeSpan = Math.Abs(((TimeSpan)(fileTime - lastScreenShot)).TotalMilliseconds);
                if (fileTimeSpan < foundFileTimeDiff &&
                    fileTimeSpan < 30000) {
                    foundFile = screenFile;
                    foundFileTimeDiff = fileTimeSpan;
                }
            }

            if (foundFile != null) return foundFile.FullName;
            return null;    // fallback
        }

        private Image MakeThumbnail(string screenFile)
        {
            Image i = Image.FromFile(screenFile);
            int w = i.Width;
            Image cropped = Crop(i, w, w / 2, AnchorPosition.None);
            Image redim = FixedSize(cropped, 256, 128);
            i.Dispose();
            cropped.Dispose();
            return redim;
        }

        private bool SaveThumbnail(Image thumb, string thumbDestDir, ImageCodecInfo codec, EncoderParameters par)
        {
            string thumbFile = thumbDestDir + "\\thumbnail.jpg";
            try
            {
                thumb.Save(thumbFile, codec, par);
                displayText("Saved thumbnail: " + thumbFile);
                return true;
            }
            catch (Exception)
            {
                return false;
            }
        }

        // **************************************************************************
        //
        // Graphics manipulation
        // Image resizing snippets from Joel Neubeck (codeproject.com)
        //
        // **************************************************************************


        [Flags]
        enum AnchorPosition
        {
            Top = 1,
            Bottom = 2,
            Left = 4,
            Right = 8,
            None = 0
        };

        static Image Crop(Image imgPhoto, int Width,
                           int Height, AnchorPosition Anchor)
        {
            int sourceWidth = imgPhoto.Width;
            int sourceHeight = imgPhoto.Height;
            int sourceX = 0;
            int sourceY = 0;
            int destX = 0;
            int destY = 0;

            float nPercent = 0;
            float nPercentW = 0;
            float nPercentH = 0;

            nPercentW = ((float)Width / (float)sourceWidth);
            nPercentH = ((float)Height / (float)sourceHeight);

            if (nPercentH < nPercentW)
            {
                nPercent = nPercentW;
                switch (Anchor)
                {
                    case AnchorPosition.Top:
                        destY = 0;
                        break;
                    case AnchorPosition.Bottom:
                        destY = (int)
                            (Height - (sourceHeight * nPercent));
                        break;
                    default:
                        destY = (int)
                            ((Height - (sourceHeight * nPercent)) / 2);
                        break;
                }
            }
            else
            {
                nPercent = nPercentH;
                switch (Anchor)
                {
                    case AnchorPosition.Left:
                        destX = 0;
                        break;
                    case AnchorPosition.Right:
                        destX = (int)
                          (Width - (sourceWidth * nPercent));
                        break;
                    default:
                        destX = (int)
                          ((Width - (sourceWidth * nPercent)) / 2);
                        break;
                }
            }

            int destWidth = (int)(sourceWidth * nPercent);
            int destHeight = (int)(sourceHeight * nPercent);

            Bitmap bmPhoto = new Bitmap(Width,
                    Height, PixelFormat.Format24bppRgb);
            bmPhoto.SetResolution(imgPhoto.HorizontalResolution,
                    imgPhoto.VerticalResolution);

            Graphics grPhoto = Graphics.FromImage(bmPhoto);
            grPhoto.InterpolationMode =
                    InterpolationMode.HighQualityBicubic;

            grPhoto.DrawImage(imgPhoto,
                new Rectangle(destX, destY, destWidth, destHeight),
                new Rectangle(sourceX, sourceY, sourceWidth, sourceHeight),
                GraphicsUnit.Pixel);

            grPhoto.Dispose();
            return bmPhoto;
        }

        static Image FixedSize(Image imgPhoto, int Width, int Height)
        {
            int sourceWidth = imgPhoto.Width;
            int sourceHeight = imgPhoto.Height;
            int sourceX = 0;
            int sourceY = 0;
            int destX = 0;
            int destY = 0;

            float nPercent = 0;
            float nPercentW = 0;
            float nPercentH = 0;

            nPercentW = ((float)Width / (float)sourceWidth);
            nPercentH = ((float)Height / (float)sourceHeight);
            if (nPercentH < nPercentW)
            {
                nPercent = nPercentH;
                destX = System.Convert.ToInt16((Width -
                              (sourceWidth * nPercent)) / 2);
            }
            else
            {
                nPercent = nPercentW;
                destY = System.Convert.ToInt16((Height -
                              (sourceHeight * nPercent)) / 2);
            }

            int destWidth = (int)(sourceWidth * nPercent);
            int destHeight = (int)(sourceHeight * nPercent);

            Bitmap bmPhoto = new Bitmap(Width, Height,
                              PixelFormat.Format24bppRgb);
            bmPhoto.SetResolution(imgPhoto.HorizontalResolution,
                             imgPhoto.VerticalResolution);

            Graphics grPhoto = Graphics.FromImage(bmPhoto);
            grPhoto.Clear(Color.White);
            grPhoto.InterpolationMode =
                    InterpolationMode.HighQualityBicubic;

            grPhoto.DrawImage(imgPhoto,
                new Rectangle(destX, destY, destWidth, destHeight),
                new Rectangle(sourceX, sourceY, sourceWidth, sourceHeight),
                GraphicsUnit.Pixel);

            grPhoto.Dispose();
            return bmPhoto;
        }


        // **************************************************************************
        //
        // Main loop
        //
        // **************************************************************************

        private void CheckFsxRunning()
        {
            Process[] sysProcs = Process.GetProcesses();
            bool fsxFound = false;
            for (int i = 0; i < sysProcs.Length; i++)
            {
                if (sysProcs[i].ProcessName.Equals("fsx", StringComparison.InvariantCultureIgnoreCase))
                {
                    fsxFound = true;
                    break;
                }
            }

            if (!fsxFound)
            {
                displayText("FSX process not found, killing");
                simconnect.Dispose();

                // huhoh, fsx went away
                // kill ourself
                Application.Exit();
            }
        }

        void doLoop()
        {
            WaitHandle[] handles = new WaitHandle[1];
            while (true)
            {
                handles[0] = waiter;
                int res = WaitHandle.WaitAny(handles, 3000, false);
                if (res == WaitHandle.WaitTimeout)
                {
                    CheckFsxRunning();
                }
                else
                {
                    simconnect.ReceiveMessage();
                }
            }
        }

        private void displayText(string s)
        {
            
            Console.WriteLine(s);
        }

        private void displayError(string s, params object[] paramlist)
        {
            if (simconnect != null)
            {
                try
                {
                    simconnect.SetSystemState("DialogMode", 1, 0f, null);
                }
                catch (COMException) { }
            }
            MessageBox.Show(String.Format(s, paramlist), "Error");
            if (simconnect != null)
            {
                try
                {
                    simconnect.SetSystemState("DialogMode", 0, 0f, null);
                }
                catch (COMException) { }
            }
        }



        static void Main(string[] args)
        {
            try
            {
                Autothumbnail app = new Autothumbnail(0);
                app.doLoop();
            }
            catch (Exception cex)
            {
                Console.WriteLine(cex);
            }
        }
    }
}
